@@ -11,6 +11,7 @@ use File::pushd;
 use CPAN::Meta;
 use Module::CPANfile;
 use Config::Identity::PAUSE;
+use Module::Runtime qw(require_module);
 
 use Minilla;
 use Minilla::Logger;
@@ -19,6 +20,7 @@ use Minilla::WorkDir;
 use Minilla::ReleaseTest;
 use Minilla::ModuleMaker::ModuleBuild;
 use Minilla::ModuleMaker::ModuleBuildTiny;
+use Minilla::ModuleMaker::ExtUtilsMakeMaker;
 use Minilla::Util qw(slurp_utf8 find_dir cmd spew_raw slurp_raw);
 use Encode qw(decode_utf8);
 
@@ -41,7 +43,7 @@ has module_maker => (
             $klass = $klass =~ s/^\+// ? $klass : "Minilla::ModuleMaker::$klass";
             return $klass->new();
         }
-        Minilla::ModuleMaker::ModuleBuild->new()
+        Minilla::ModuleMaker::ModuleBuildTiny->new()
     },
     lazy => 1,
 );
@@ -223,12 +225,35 @@ sub config {
         if ($err) {
             errorf("TOML error in %s: %s\n", $toml_path, $err);
         }
+        $self->_patch_config_for_mb($conf) unless $conf->{module_maker};
         $conf;
     } else {
         +{};
     }
 }
 
+sub _patch_config_for_mb {
+    my($self, $conf) = @_;
+
+    if (exists $conf->{build} or exists $conf->{XSUtil}) {
+        warn <<WARN unless $self->{__already_warned}++;
+!
+! WARNING:
+! module_maker is not set in your Minilla config (minil.toml), but found [build] or [XSUtil] section in it.
+! Defaulting to Module::Build, but you're suggested to add the following to your minil.toml:
+!
+!   module_maker="ModuleBuild"
+!
+! This friendly warning will go away in the next major release, and Minilla will default to ModuleBuildTiny
+! when module_maker is not explicitly set in minil.toml.
+!
+WARN
+        $conf->{module_maker} = "ModuleBuild";
+    }
+
+    return;
+}
+
 sub c_source {
     my $self = shift;
     $self->config->{c_source} ? join(' ', @{$self->config->{c_source}}) : '';
@@ -468,13 +493,17 @@ sub extract_git_info {
     my $homepage;
     if ( `git remote show -n origin` =~ /URL: (.*)$/m && $1 ne 'origin' ) {
         # XXX Make it public clone URL, but this only works with github
-        my $git_url = $1;
-        $git_url =~ s![\w\-]+\@([^:]+):!git://$1/!;
-        if ($git_url =~ /github\.com/) {
-            my $http_url = $git_url;
-            $http_url =~ s![\w\-]+\@([^:]+):!https://$1/!;
-            $http_url =~ s!\Agit://!https://!;
-            $http_url =~ s!\.git$!!;
+        my $registered_url = $1;
+        if ($registered_url =~ /github\.com/) {
+            my ($user, $repo) = $registered_url =~ m{
+                github\.com
+                (?:(?::[0-9]+)?/|:)([^/]+)
+                /
+                ([^/]+?)(?:\.git)?
+                $
+            }ix;
+            my $git_url = "git://github.com/$user/$repo.git";
+            my $http_url = "https://github.com/$user/$repo";
             unless ($self->config->{no_github_issues}) {
                 $bugtracker = +{
                     web => "$http_url/issues",
@@ -486,10 +515,25 @@ sub extract_git_info {
             };
             $homepage = $self->config->{homepage} || $http_url;
         } else {
+            if ($registered_url !~ m{^(?:https?|ssh|git)://}) {
+                # We can't do much more than this, but we need to fix
+                # user@host:path/to/repo.git to git://$host/path/to/repo.git in
+                # order to work with CPAN::Meta
+                $registered_url =~ s{
+                    \A
+                    [^@]+       # user name, which we toss away
+                    @
+                    ([^:]+)     # anything other than a ":"
+                    :
+                    (.+)        # anything, which is the repository
+                    \Z
+                }{git://$1/$2}gx;
+            }
+
             # normal repository
-            if ($git_url !~ m{^file://}) {
+            if ($registered_url !~ m{^file://}) {
                 $repository = +{
-                    url => $git_url,
+                    url => $registered_url,
                 };
             }
         }
@@ -547,8 +591,9 @@ sub generate_minil_toml {
 
     if ($profile eq 'ModuleBuild') {
         $content .= qq{\nmodule_maker="ModuleBuild"\n};
-    }
-    else {
+    } elsif ($profile eq 'ExtUtilsMakeMaker') {
+        $content .= qq{\nmodule_maker="ExtUtilsMakeMaker"\n};
+    } else {
         $content .= qq{\nmodule_maker="ModuleBuildTiny"\n};
     }
 
@@ -558,10 +603,16 @@ sub generate_minil_toml {
 sub regenerate_readme_md {
     my $self = shift;
 
-    require Pod::Markdown;
-    Pod::Markdown->VERSION('1.322');
+    my $markdown_maker = $self->config->{markdown_maker} || 'Pod::Markdown';
+    require_module($markdown_maker);
+    if ($markdown_maker eq 'Pod::Markdown') {
+        $markdown_maker->VERSION('1.322');
+    }
 
-    my $parser = Pod::Markdown->new;
+    my $parser = $markdown_maker->new;
+    if (not $parser->isa('Pod::Markdown')) {
+        errorf("'markdown_maker' config key must be a subclass of Pod::Markdown\n");
+    }
     $parser->parse_from_file($self->readme_from);
 
     my $fname = File::Spec->catfile($self->dir, 'README.md');
@@ -580,10 +631,13 @@ sub regenerate_readme_md {
         if ($user_name && $repository_name) {
             for my $badge (@{$self->badges}) {
                 if ($badge eq 'travis') {
-                    push @badges, "[![Build Status](https://travis-ci.org/$user_name/$repository_name.png?branch=master)](https://travis-ci.org/$user_name/$repository_name)";
+                    push @badges, "[![Build Status](https://travis-ci.org/$user_name/$repository_name.svg?branch=master)](https://travis-ci.org/$user_name/$repository_name)";
                 }
                 if ($badge eq 'coveralls') {
-                    push @badges, "[![Coverage Status](https://coveralls.io/repos/$user_name/$repository_name/badge.png?branch=master)](https://coveralls.io/r/$user_name/$repository_name?branch=master)"
+                    push @badges, "[![Coverage Status](https://img.shields.io/coveralls/$user_name/$repository_name/master.svg?style=flat)](https://coveralls.io/r/$user_name/$repository_name?branch=master)"
+                }
+                if ($badge eq 'gitter') {
+                    push @badges, "[![Gitter chat](https://badges.gitter.im/$user_name/$repository_name.png)](https://gitter.im/$user_name/$repository_name)";
                 }
             }
         }
@@ -606,6 +660,8 @@ sub verify_prereqs {
 sub _build_contributors {
     my $self = shift;
 
+    return [] unless (`git show-ref --head`);
+
     my $normalize = sub {
         local $_ = shift;
         if (/<([^>]+)>/) {
@@ -660,4 +716,15 @@ sub requires_external_bin {
     return $self->config->{requires_external_bin};
 }
 
+# @return true if the project is valid, false otherwise.
+sub validate {
+    my $self = shift;
+    my $module_maker = $self->module_maker;
+    if ($module_maker->can('validate')) {
+        return $module_maker->validate();
+    } else {
+        return 1;
+    }
+}
+
 1;
